use std::env;
use config::{Config, ConfigError, File};
use serde::Deserialize;

/// Application configuration
#[derive(Debug, Deserialize, Clone)]
pub struct AppConfig {
    /// Server configuration
    pub server: ServerConfig,
    /// Template configuration
    pub templates: TemplateConfig,
}

/// Server configuration
#[derive(Debug, Deserialize, Clone)]
pub struct ServerConfig {
    /// Host address to bind to
    pub host: String,
    /// Port to listen on
    pub port: u16,
    /// Workers count
    pub workers: Option<u32>,
}

/// Template configuration
#[derive(Debug, Deserialize, Clone)]
pub struct TemplateConfig {
    /// Directory containing templates
    pub dir: String,
}

impl AppConfig {
    /// Loads configuration from files and environment variables
    pub fn new() -> Result<Self, ConfigError> {
        // Set default values
        let mut config_builder = Config::builder()
            .set_default("server.host", "127.0.0.1")?
            .set_default("server.port", 9999)?
            .set_default("server.workers", None::<u32>)?
            .set_default("templates.dir", "./src/views")?;

        // Load from config file if it exists
        if let Ok(config_path) = env::var("APP_CONFIG") {
            config_builder = config_builder.add_source(File::with_name(&config_path));
        } else {
            // Try to load from default locations
            config_builder = config_builder
                .add_source(File::with_name("config/default").required(false))
                .add_source(File::with_name("config/local").required(false));
        }

        // Override with environment variables (e.g., SERVER__HOST, SERVER__PORT)
        config_builder = config_builder.add_source(config::Environment::with_prefix("APP").separator("__"));

        // Build and deserialize the config
        let config = config_builder.build()?;
        config.try_deserialize()
    }
}

/// Returns the application configuration
pub fn get_config() -> AppConfig {
    AppConfig::new().expect("Failed to load configuration")
}